@@ -1,28 +1,3 @@
-val gitRevision: String by rootProject.extra
-val apktoolVersion: String by rootProject.extra
-
-tasks {
-    processResources {
-        from("src/main/resources") {
-            include("apktool.properties")
-            expand("version" to apktoolVersion, "gitrev" to gitRevision)
-            duplicatesStrategy = DuplicatesStrategy.INCLUDE
-        }
-        from("src/main/resources") {
-            include("**/*.jar")
-            duplicatesStrategy = DuplicatesStrategy.INCLUDE
-        }
-        includeEmptyDirs = false
-    }
-
-    test {
-        // https://github.com/iBotPeaches/Apktool/issues/3174 - CVE-2023-22036
-        // Increases validation of extra field of zip header. Some older Android applications
-        // used this field to store data violating the zip specification.
-        systemProperty("jdk.util.zip.disableZip64ExtraFieldValidation", true)
-    }
-}
-
 dependencies {
     api(project(":brut.j.common"))
     api(project(":brut.j.util"))
@@ -40,3 +15,24 @@ dependencies {
     testImplementation(libs.junit)
     testImplementation(libs.xmlunit)
 }
+
+tasks {
+    processResources {
+        from("src/main/resources") {
+            include("**/*.jar")
+            duplicatesStrategy = DuplicatesStrategy.INCLUDE
+        }
+        includeEmptyDirs = false
+    }
+
+    test {
+        // https://github.com/iBotPeaches/Apktool/issues/3174 - CVE-2023-22036
+        // Increases validation of extra field of zip header. Some older Android apps
+        // used this field to store data violating the zip specification.
+        systemProperty("jdk.util.zip.disableZip64ExtraFieldValidation", true)
+
+        // Fix for AWT/X11 graphics environment issues in headless environments
+        // Required for tests that use ImageIO operations (nine-patch processing, etc.)
+        systemProperty("java.awt.headless", true)
+    }
+}
