@@ -20,65 +20,66 @@ import brut.androlib.exceptions.AndrolibException;
 import brut.androlib.exceptions.InFileNotFoundException;
 import brut.androlib.exceptions.OutDirExistsException;
 import brut.androlib.meta.ApkInfo;
-import brut.androlib.res.ResourcesDecoder;
-import brut.androlib.src.SmaliDecoder;
-import brut.common.BrutException;
+import brut.androlib.res.ResDecoder;
+import brut.androlib.smali.SmaliDecoder;
+import brut.common.Log;
 import brut.directory.Directory;
 import brut.directory.DirectoryException;
 import brut.directory.ExtFile;
 import brut.util.BackgroundWorker;
 import brut.util.OS;
-import com.android.tools.smali.dexlib2.iface.DexFile;
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.commons.io.FilenameUtils;
 
 import java.io.*;
 import java.util.*;
 import java.util.concurrent.atomic.AtomicReference;
-import java.util.logging.Logger;
 import java.util.regex.Pattern;
 
 public class ApkDecoder {
-    private static final Logger LOGGER = Logger.getLogger(ApkDecoder.class.getName());
+    private static final String TAG = ApkDecoder.class.getName();
 
-    // extensions of files that are often packed uncompressed
     private static final Pattern NO_COMPRESS_EXT_PATTERN = Pattern.compile(
         "dex|arsc|so|jpg|jpeg|png|gif|wav|mp2|mp3|ogg|aac|mpg|mpeg|mid|midi|smf|jet|"
-            + "rtttl|imy|xmf|mp4|m4a|m4v|3gp|3gpp|3g2|3gpp2|amr|awb|wma|wmv|webm|webp|mkv");
+      + "rtttl|imy|xmf|mp4|m4a|m4v|3gp|3gpp|3g2|3gpp2|amr|awb|wma|wmv|webm|webp|mkv");
 
     private final ExtFile mApkFile;
     private final Config mConfig;
-    private final AtomicReference<AndrolibException> mBuildError;
+    private final AtomicReference<AndrolibException> mFirstError;
 
     private ApkInfo mApkInfo;
-    private ResourcesDecoder mResDecoder;
-    private volatile int mMinSdkVersion;
+    private SmaliDecoder mSmaliDecoder;
+    private ResDecoder mResDecoder;
     private BackgroundWorker mWorker;
 
-    public ApkDecoder(ExtFile apkFile, Config config) {
-        mApkFile = apkFile;
+    public ApkDecoder(File apkFile, Config config) {
+        mApkFile = new ExtFile(apkFile);
         mConfig = config;
-        mBuildError = new AtomicReference<>(null);
+        mFirstError = new AtomicReference<>();
     }
 
-    public ApkInfo decode(File outDir) throws AndrolibException {
+    public void decode(File outDir) throws AndrolibException {
         if (!mConfig.isForced() && outDir.exists()) {
-            throw new OutDirExistsException();
+            throw new OutDirExistsException(outDir.getPath());
         }
         if (!mApkFile.isFile() || !mApkFile.canRead()) {
-            throw new InFileNotFoundException();
+            throw new InFileNotFoundException(mApkFile.getPath());
         }
         if (mConfig.getJobs() > 1) {
             mWorker = new BackgroundWorker(mConfig.getJobs() - 1);
         }
         try {
-            mApkInfo = new ApkInfo(mApkFile);
-            mResDecoder = new ResourcesDecoder(mApkInfo, mConfig);
+            mApkInfo = new ApkInfo();
+            mApkInfo.setVersion(mConfig.getVersion());
+            mApkInfo.setApkFile(mApkFile);
+            mSmaliDecoder = new SmaliDecoder(mApkFile, mConfig.isBaksmaliDebugMode());
+            mResDecoder = new ResDecoder(mApkInfo, mConfig);
 
             OS.rmdir(outDir);
             OS.mkdir(outDir);
 
-            LOGGER.info("Using Apktool " + ApktoolProperties.getVersion() + " on " + mApkFile.getName()
-                        + (mWorker != null ? " with " + mConfig.getJobs() + " threads" : ""));
+            Log.i(TAG, "Using Apktool " + mConfig.getVersion() + " on " + mApkFile.getName()
+                     + (mWorker != null ? " with " + mConfig.getJobs() + " threads" : ""));
 
             decodeSources(outDir);
             decodeResources(outDir);
@@ -86,8 +87,8 @@ public class ApkDecoder {
 
             if (mWorker != null) {
                 mWorker.waitForFinish();
-                if (mBuildError.get() != null) {
-                    throw mBuildError.get();
+                if (mFirstError.get() != null) {
+                    throw mFirstError.get();
                 }
             }
 
@@ -95,54 +96,41 @@ public class ApkDecoder {
             copyRawFiles(outDir);
             copyUnknownFiles(outDir);
             writeApkInfo(outDir);
-
-            return mApkInfo;
         } finally {
             if (mWorker != null) {
                 mWorker.shutdownNow();
             }
             try {
                 mApkFile.close();
-            } catch (IOException ignored) {}
+            } catch (DirectoryException ignored) {
+            }
         }
     }
 
+    @VisibleForTesting
+    ApkInfo getApkInfo() {
+        return mApkInfo;
+    }
+
     private void decodeSources(File outDir) throws AndrolibException {
         if (!mApkInfo.hasSources()) {
             return;
         }
 
-        switch (mConfig.getDecodeSources()) {
-            case NONE:
-                copySourcesRaw(outDir, "classes.dex");
-                break;
-            case FULL:
-            case ONLY_MAIN_CLASSES:
-                decodeSourcesSmali(outDir, "classes.dex");
-                break;
-        }
-
         try {
             Directory in = mApkFile.getDirectory();
+            boolean allSrc = mConfig.isDecodeSourcesFull();
+            boolean noSrc = mConfig.isDecodeSourcesNone();
 
-            // foreach unknown dex file in root, lets disassemble it
-            for (String fileName : in.getFiles(true)) {
-                if (fileName.endsWith(".dex") && !fileName.equals("classes.dex")) {
-                    switch (mConfig.getDecodeSources()) {
-                        case NONE:
-                            copySourcesRaw(outDir, fileName);
-                            break;
-                        case FULL:
-                            decodeSourcesSmali(outDir, fileName);
-                            break;
-                        case ONLY_MAIN_CLASSES:
-                            if (fileName.startsWith("classes")) {
-                                decodeSourcesSmali(outDir, fileName);
-                            } else {
-                                copySourcesRaw(outDir, fileName);
-                            }
-                            break;
-                    }
+            for (String fileName : in.getFiles(allSrc)) {
+                if (allSrc ? !fileName.endsWith(".dex") : !ApkInfo.CLASSES_FILES_PATTERN.matcher(fileName).matches()) {
+                    continue;
+                }
+
+                if (noSrc) {
+                    copySourcesRaw(outDir, fileName);
+                } else {
+                    decodeSourcesSmali(outDir, fileName);
                 }
             }
         } catch (DirectoryException ex) {
@@ -151,7 +139,7 @@ public class ApkDecoder {
     }
 
     private void copySourcesRaw(File outDir, String fileName) throws AndrolibException {
-        LOGGER.info("Copying raw " + fileName + " file...");
+        Log.i(TAG, "Copying raw " + fileName + "...");
         try {
             Directory in = mApkFile.getDirectory();
 
@@ -164,11 +152,11 @@ public class ApkDecoder {
     private void decodeSourcesSmali(File outDir, String fileName) throws AndrolibException {
         if (mWorker != null) {
             mWorker.submit(() -> {
-                if (mBuildError.get() == null) {
+                if (mFirstError.get() == null) {
                     try {
                         decodeSourcesSmaliJob(outDir, fileName);
                     } catch (AndrolibException ex) {
-                        mBuildError.compareAndSet(null, ex);
+                        mFirstError.compareAndSet(null, ex);
                     }
                 }
             });
@@ -178,26 +166,8 @@ public class ApkDecoder {
     }
 
     private void decodeSourcesSmaliJob(File outDir, String fileName) throws AndrolibException {
-        File smaliDir;
-        if (fileName.equals("classes.dex")) {
-            smaliDir = new File(outDir, "smali");
-        } else {
-            smaliDir = new File(outDir, "smali_" + fileName.substring(0, fileName.indexOf(".")));
-        }
-
-        OS.rmdir(smaliDir);
-        OS.mkdir(smaliDir);
-
-        LOGGER.info("Baksmaling " + fileName + "...");
-        SmaliDecoder decoder = new SmaliDecoder(mApkFile, fileName,
-            mConfig.isBaksmaliDebugMode(), mConfig.getBaksmaliApiLevel());
-        DexFile dexFile = decoder.decode(smaliDir);
-
-        // record minSdkVersion for jars
-        int minSdkVersion = dexFile.getOpcodes().api;
-        if (mMinSdkVersion == 0 || mMinSdkVersion > minSdkVersion) {
-            mMinSdkVersion = minSdkVersion;
-        }
+        Log.i(TAG, "Baksmaling " + fileName + "...");
+        mSmaliDecoder.decode(fileName, outDir);
     }
 
     private void decodeResources(File outDir) throws AndrolibException {
@@ -205,24 +175,19 @@ public class ApkDecoder {
             return;
         }
 
-        switch (mConfig.getDecodeResources()) {
-            case NONE:
-            case ONLY_MANIFEST:
-                copyResourcesRaw(outDir);
-                break;
-            case FULL:
-                mResDecoder.decodeResources(outDir);
-                break;
+        if (mConfig.isDecodeResourcesFull()) {
+            mResDecoder.decodeResources(outDir);
+        } else {
+            copyResourcesRaw(outDir);
         }
     }
 
     private void copyResourcesRaw(File outDir) throws AndrolibException {
-        LOGGER.info("Copying raw resources...");
+        Log.i(TAG, "Copying raw resources.arsc...");
         try {
             Directory in = mApkFile.getDirectory();
 
             in.copyToDir(outDir, "resources.arsc");
-            in.copyToDir(outDir, ApkInfo.RESOURCES_DIRNAMES);
         } catch (DirectoryException ex) {
             throw new AndrolibException(ex);
         }
@@ -233,19 +198,15 @@ public class ApkDecoder {
             return;
         }
 
-        switch (mConfig.getDecodeResources()) {
-            case NONE:
-                copyManifestRaw(outDir);
-                break;
-            case FULL:
-            case ONLY_MANIFEST:
-                mResDecoder.decodeManifest(outDir);
-                break;
+        if (!mConfig.isDecodeResourcesNone()) {
+            mResDecoder.decodeManifest(outDir);
+        } else {
+            copyManifestRaw(outDir);
         }
     }
 
     private void copyManifestRaw(File outDir) throws AndrolibException {
-        LOGGER.info("Copying raw manifest...");
+        Log.i(TAG, "Copying raw AndroidManifest.xml...");
         try {
             Directory in = mApkFile.getDirectory();
 
@@ -258,16 +219,21 @@ public class ApkDecoder {
     private void copyRawFiles(File outDir) throws AndrolibException {
         try {
             Directory in = mApkFile.getDirectory();
+            Set<String> dexFiles = mSmaliDecoder.getDexFiles();
+            Map<String, String> resFileMap = mResDecoder.getResFileMap();
+            boolean noAssets = mConfig.isDecodeAssetsNone();
 
-            for (String dirName : ApkInfo.RAW_DIRNAMES) {
-                if ((mConfig.getDecodeAssets() == Config.DecodeAssets.FULL || !dirName.equals("assets"))
-                        && in.containsDir(dirName)) {
-                    LOGGER.info("Copying " + dirName + "...");
-                    for (String fileName : in.getDir(dirName).getFiles(true)) {
-                        fileName = dirName + "/" + fileName;
-                        if (!ApkInfo.ORIGINAL_FILENAMES_PATTERN.matcher(fileName).matches()) {
-                            in.copyToDir(outDir, fileName);
-                        }
+            for (String dirName : ApkInfo.RAW_DIRS) {
+                if (!in.containsDir(dirName) || (noAssets && dirName.equals("assets"))) {
+                    continue;
+                }
+
+                Log.i(TAG, "Copying " + dirName + "...");
+                for (String fileName : in.getDir(dirName).getFiles(true)) {
+                    fileName = dirName + in.separator + fileName;
+                    if (!ApkInfo.ORIGINAL_FILES_PATTERN.matcher(fileName).matches()
+                            && !dexFiles.contains(fileName) && !resFileMap.containsKey(fileName)) {
+                        in.copyToDir(outDir, fileName);
                     }
                 }
             }
@@ -277,13 +243,14 @@ public class ApkDecoder {
     }
 
     private void copyOriginalFiles(File outDir) throws AndrolibException {
-        LOGGER.info("Copying original files...");
+        File originalDir = new File(outDir, "original");
+
+        Log.i(TAG, "Copying original files...");
         try {
             Directory in = mApkFile.getDirectory();
-            File originalDir = new File(outDir, "original");
 
             for (String fileName : in.getFiles(true)) {
-                if (ApkInfo.ORIGINAL_FILENAMES_PATTERN.matcher(fileName).matches()) {
+                if (ApkInfo.ORIGINAL_FILES_PATTERN.matcher(fileName).matches()) {
                     in.copyToDir(originalDir, fileName);
                 }
             }
@@ -293,13 +260,17 @@ public class ApkDecoder {
     }
 
     private void copyUnknownFiles(File outDir) throws AndrolibException {
-        LOGGER.info("Copying unknown files...");
+        File unknownDir = new File(outDir, "unknown");
+
+        Log.i(TAG, "Copying unknown files...");
         try {
             Directory in = mApkFile.getDirectory();
-            File unknownDir = new File(outDir, "unknown");
+            Set<String> dexFiles = mSmaliDecoder.getDexFiles();
+            Map<String, String> resFileMap = mResDecoder.getResFileMap();
 
             for (String fileName : in.getFiles(true)) {
-                if (!ApkInfo.STANDARD_FILENAMES_PATTERN.matcher(fileName).matches()) {
+                if (!ApkInfo.STANDARD_FILES_PATTERN.matcher(fileName).matches() && !dexFiles.contains(fileName)
+                        && !resFileMap.containsKey(fileName)) {
                     in.copyToDir(unknownDir, fileName);
                 }
             }
@@ -309,32 +280,34 @@ public class ApkDecoder {
     }
 
     private void writeApkInfo(File outDir) throws AndrolibException {
-        // in case we have no resources, we should store the minSdk we pulled from the source opcode api level
-        if (!mApkInfo.hasResources() && mMinSdkVersion > 0) {
-            mApkInfo.getSdkInfo().setMinSdkVersion(Integer.toString(mMinSdkVersion));
+        // If we did not decode the manifest, store the inferred dex opcode API level.
+        if (!mApkInfo.hasManifest() || mConfig.isDecodeResourcesNone()) {
+            int apiLevel = mSmaliDecoder.getInferredApiLevel();
+            if (apiLevel > 0) {
+                mApkInfo.getSdkInfo().setMinSdkVersion(Integer.toString(apiLevel));
+            }
         }
 
-        // record uncompressed files
+        // Record uncompressed files.
         try {
-            Map<String, String> resFileMapping = mResDecoder.getResFileMapping();
+            Directory in = mApkFile.getDirectory();
+            Map<String, String> resFileMap = mResDecoder.getResFileMap();
             Set<String> uncompressedExts = new HashSet<>();
             Set<String> uncompressedFiles = new HashSet<>();
-            Directory in = mApkFile.getDirectory();
 
             for (String fileName : in.getFiles(true)) {
                 if (in.getCompressionLevel(fileName) == 0) {
                     String ext;
-                    if (in.getSize(fileName) > 0
-                            && !(ext = FilenameUtils.getExtension(fileName)).isEmpty()
+                    if (in.getSize(fileName) > 0 && !(ext = FilenameUtils.getExtension(fileName)).isEmpty()
                             && NO_COMPRESS_EXT_PATTERN.matcher(ext).matches()) {
                         uncompressedExts.add(ext);
                     } else {
-                        uncompressedFiles.add(resFileMapping.getOrDefault(fileName, fileName));
+                        uncompressedFiles.add(resFileMap.getOrDefault(fileName, fileName));
                     }
                 }
             }
 
-            // exclude files with an already recorded extenstion
+            // Exclude files with an already recorded extenstion.
             if (!uncompressedExts.isEmpty() && !uncompressedFiles.isEmpty()) {
                 Iterator<String> it = uncompressedFiles.iterator();
                 while (it.hasNext()) {
@@ -346,7 +319,7 @@ public class ApkDecoder {
                 }
             }
 
-            // update apk info
+            // Update apk info.
             List<String> doNotCompress = mApkInfo.getDoNotCompress();
             if (!uncompressedExts.isEmpty()) {
                 List<String> uncompressedExtsList = new ArrayList<>(uncompressedExts);
@@ -362,7 +335,7 @@ public class ApkDecoder {
             throw new AndrolibException(ex);
         }
 
-        // write apk info to file
-        mApkInfo.save(new File(outDir, "apktool.yml"));
+        // Serialize apk info to file.
+        mApkInfo.save(outDir);
     }
 }
