@@ -18,7 +18,7 @@ package brut.androlib.exceptions;
 
 public class RawXmlEncounteredException extends AndrolibException {
 
-    public RawXmlEncounteredException(String message, Throwable cause) {
-        super(message, cause);
+    public RawXmlEncounteredException(Throwable cause) {
+        super("Could not decode XML.", cause);
     }
 }
