@@ -19,7 +19,9 @@ package brut.androlib.res.decoder;
 import brut.androlib.exceptions.AndrolibException;
 import org.apache.commons.io.IOUtils;
 
-import java.io.*;
+import java.io.InputStream;
+import java.io.IOException;
+import java.io.OutputStream;
 
 public class ResRawStreamDecoder implements ResStreamDecoder {
 
@@ -28,7 +30,7 @@ public class ResRawStreamDecoder implements ResStreamDecoder {
         try {
             IOUtils.copy(in, out);
         } catch (IOException ex) {
-            throw new AndrolibException("Could not decode raw stream", ex);
+            throw new AndrolibException("Could not decode raw stream.", ex);
         }
     }
 }
