@@ -16,26 +16,23 @@
  */
 package brut.androlib;
 
-import brut.common.BrutException;
-import brut.directory.ExtFile;
-
 import java.io.File;
 
 import org.junit.*;
 import static org.junit.Assert.*;
 
 public class Empty9PatchTest extends BaseTest {
-    private static final String TEST_APK = "empty9patch.apk";
+    private static final String TEST_APK = "empty_9patch.apk";
 
     @BeforeClass
     public static void beforeClass() throws Exception {
-        TestUtils.copyResourceDir(Empty9PatchTest.class, "empty9patch", sTmpDir);
+        copyResourceDir(Empty9PatchTest.class, "empty_9patch", sTmpDir);
     }
 
     @Test
-    public void decodeWithEmpty9PatchFile() throws BrutException {
-        ExtFile testApk = new ExtFile(sTmpDir, TEST_APK);
-        ExtFile testDir = new ExtFile(testApk + ".out");
+    public void decodeWithEmpty9PatchFile() throws Exception {
+        File testApk = new File(sTmpDir, TEST_APK);
+        File testDir = new File(testApk + ".out");
         new ApkDecoder(testApk, sConfig).decode(testDir);
 
         File aPng = new File(testDir, "res/drawable-xhdpi/empty.9.png");
