@@ -16,8 +16,7 @@
  */
 package brut.androlib;
 
-import brut.common.BrutException;
-import brut.directory.ExtFile;
+import java.io.File;
 
 import org.junit.*;
 import static org.junit.Assert.*;
@@ -27,16 +26,15 @@ public class ParentDirectoryTraversalTest extends BaseTest {
 
     @BeforeClass
     public static void beforeClass() throws Exception {
-        TestUtils.copyResourceDir(ParentDirectoryTraversalTest.class, "issue1498", sTmpDir);
+        copyResourceDir(ParentDirectoryTraversalTest.class, "issue1498", sTmpDir);
     }
 
     @Test
-    public void checkIfDrawableFileDecodesProperly() throws BrutException {
-        sConfig.setForced(true);
+    public void checkIfDrawableFileDecodesProperly() throws Exception {
         sConfig.setDecodeResources(Config.DecodeResources.NONE);
 
-        ExtFile testApk = new ExtFile(sTmpDir, apk);
-        ExtFile testDir = new ExtFile(testApk + ".out");
+        File testApk = new File(sTmpDir, apk);
+        File testDir = new File(testApk + ".out");
         new ApkDecoder(testApk, sConfig).decode(testDir);
     }
 }
