@@ -22,23 +22,17 @@ import java.io.File;
 import java.io.InputStream;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.io.UnsupportedEncodingException;
-import java.net.URLDecoder;
 import java.nio.file.Files;
 import java.util.Arrays;
 import java.util.Comparator;
 import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
 
-public class FileDirectory extends AbstractDirectory {
+public class FileDirectory extends Directory {
     private final File mDir;
 
-    public FileDirectory(ExtFile dir, String folder) throws DirectoryException {
-        this(new File(dir.toString().replaceAll("%20", " "), folder));
-    }
-
-    public FileDirectory(String dir) throws DirectoryException, UnsupportedEncodingException {
-        this(new File(URLDecoder.decode(dir, "UTF-8")));
+    public FileDirectory(String dirName) throws DirectoryException {
+        this(new File(dirName));
     }
 
     public FileDirectory(File dir) throws DirectoryException {
@@ -49,28 +43,31 @@ public class FileDirectory extends AbstractDirectory {
     }
 
     @Override
-    public long getSize(String fileName) throws DirectoryException {
-        File file = new File(generatePath(fileName));
-        if (!file.isFile()) {
-            throw new DirectoryException("file must be a file: " + file);
+    protected void load() {
+        mFiles = new LinkedHashSet<>();
+        mDirs = new LinkedHashMap<>();
+
+        File[] files = mDir.listFiles();
+        Arrays.sort(files, Comparator.comparing(File::getName));
+
+        for (File file : files) {
+            if (file.isFile()) {
+                mFiles.add(file.getName());
+            } else {
+                try {
+                    mDirs.put(file.getName(), new FileDirectory(file));
+                } catch (DirectoryException ignored) {
+                }
+            }
         }
-        return file.length();
+    }
+
+    private String generatePath(String name) {
+        return mDir.getPath() + separator + name;
     }
 
     @Override
-    public long getCompressedSize(String fileName) throws DirectoryException {
-        return getSize(fileName);
-    }
-
-    @Override
-    protected AbstractDirectory createDirLocal(String name) throws DirectoryException {
-        File dir = new File(generatePath(name));
-        OS.mkdir(dir);
-        return new FileDirectory(dir);
-    }
-
-    @Override
-    protected InputStream getFileInputLocal(String name) throws DirectoryException {
+    protected InputStream getFileInputImpl(String name) throws DirectoryException {
         try {
             File file = new File(generatePath(name));
             return Files.newInputStream(file.toPath());
@@ -80,7 +77,7 @@ public class FileDirectory extends AbstractDirectory {
     }
 
     @Override
-    protected OutputStream getFileOutputLocal(String name) throws DirectoryException {
+    protected OutputStream getFileOutputImpl(String name) throws DirectoryException {
         try {
             File file = new File(generatePath(name));
             return Files.newOutputStream(file.toPath());
@@ -90,44 +87,34 @@ public class FileDirectory extends AbstractDirectory {
     }
 
     @Override
-    protected void loadDirs() {
-        loadAll();
-    }
-
-    @Override
-    protected void loadFiles() {
-        loadAll();
-    }
-
-    @Override
-    protected void removeFileLocal(String name) {
+    protected void removeFileImpl(String name) {
         File file = new File(generatePath(name));
         OS.rmfile(file);
     }
 
-    private String generatePath(String name) {
-        return getDir().getPath() + separator + name;
+    @Override
+    protected Directory createDirImpl(String name) throws DirectoryException {
+        File dir = new File(generatePath(name));
+        OS.mkdir(dir);
+        return new FileDirectory(dir);
     }
 
-    private void loadAll() {
-        mFiles = new LinkedHashSet<>();
-        mDirs = new LinkedHashMap<>();
-
-        File[] files = getDir().listFiles();
-        Arrays.sort(files, Comparator.comparing(File::getName));
-
-        for (File file : files) {
-            if (file.isFile()) {
-                mFiles.add(file.getName());
-            } else {
-                try {
-                    mDirs.put(file.getName(), new FileDirectory(file));
-                } catch (DirectoryException ignored) {}
-            }
+    @Override
+    public long getSize(String name) throws DirectoryException {
+        File file = new File(generatePath(name));
+        if (!file.isFile()) {
+            throw new DirectoryException("file must be a file: " + file);
         }
+        return file.length();
     }
 
-    public File getDir() {
-        return mDir;
+    @Override
+    public long getCompressedSize(String name) throws DirectoryException {
+        return getSize(name);
+    }
+
+    @Override
+    public int getCompressionLevel(String name) {
+        return 0;
     }
 }
